@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 by Barry A. Warsaw
+# Copyright (C) 2007-2012 by Barry A. Warsaw
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public License
@@ -46,6 +46,8 @@ data.  In a distributed (NFS) environment, you also need to make sure that
 your clocks are properly synchronized.
 """
 
+from __future__ import absolute_import, print_function, unicode_literals
+
 __metaclass__ = type
 __all__ = [
     'LockError',
@@ -125,18 +127,11 @@ class Lock:
             lifetime = DEFAULT_LOCK_LIFETIME
         self._lockfile = lockfile
         self._lifetime = lifetime
-        # Calculate a hard link file name that will be used to lay claim to
-        # the lock.  We need to watch out for two Lock objects in the same
-        # process pointing to the same lock file.  Without this, if you lock
-        # lf1 and do not lock lf2, lf2.locked() will still return True.
-        self._claimfile = SEP.join((
-            self._lockfile,
-            socket.getfqdn(),
-            str(os.getpid()),
-            str(random.randint(0, MAXINT)),
-            ))
+        self._set_claimfile()
         # For transferring ownership across a fork.
         self._owned = True
+        # For extending the set of NFS errnos that are retried in _read().
+        self._retry_errnos = []
 
     def __repr__(self):
         return '<%s %s [%s: %s] pid=%s at %#xx>' % (
@@ -162,7 +157,7 @@ class Lock:
             raise
         # Rearrange for signature.
         try:
-            lockfile, hostname, pid, random = filename.split(SEP)
+            lockfile, hostname, pid, random_ignored = filename.split(SEP)
         except ValueError:
             raise NotLockedError('Details are unavailable')
         return hostname, int(pid), lockfile
@@ -192,7 +187,7 @@ class Lock:
             self._lifetime = lifetime
         # Do we have the lock?  As a side effect, this refreshes the lock!
         if not self.is_locked and not unconditionally:
-            raise NotLockedError('%s: %s' % (repr(self), self._read()))
+            raise NotLockedError('{0}: {1}'.format(repr(self), self._read()))
 
     def lock(self, timeout=None):
         """Acquire the lock.
@@ -217,7 +212,7 @@ class Lock:
         # very rare occurrence, only happens from cron, and has only(?) been
         # observed on Solaris 2.6.
         self._touch()
-        log.debug('laying claim: %s', self._lockfile)
+        log.debug('laying claim: {0}'.format(self._lockfile))
         # For quieting the logging output
         loopcount = -1
         while True:
@@ -228,7 +223,7 @@ class Lock:
                 # If we got here, we know we know we got the lock, and never
                 # had it before, so we're done.  Just touch it again for the
                 # fun of it.
-                log.debug('got the lock: %s', self._lockfile)
+                log.debug('got the lock: {0}'.format(self._lockfile))
                 self._touch()
                 break
             except OSError as error:
@@ -254,10 +249,11 @@ class Lock:
                 elif self._linkcount != 2:
                     # Somebody's messin' with us!  Log this, and try again
                     # later.  XXX should we raise an exception?
-                    log.error('unexpected linkcount: %d', self._linkcount)
+                    log.error('unexpected linkcount: {0:d}'.format(
+                        self._linkcount))
                 elif self._read() == self._claimfile:
                     # It was us that already had the link.
-                    log.debug('already locked: %s', self._lockfile)
+                    log.debug('already locked: {0}'.format(self._lockfile))
                     raise AlreadyLockedError('We already had the lock')
                 # otherwise, someone else has the lock
                 pass
@@ -280,7 +276,7 @@ class Lock:
             # and the expected lock lifetime hasn't expired yet either.  So
             # let's wait a while for the owner of the lock to give it up.
             elif not loopcount % 100:
-                log.debug('waiting for claim: %s', self._lockfile)
+                log.debug('waiting for claim: {0}'.format(self._lockfile))
             self._sleep()
 
     def unlock(self, unconditionally=False):
@@ -308,7 +304,7 @@ class Lock:
         except OSError as error:
             if error.errno != errno.ENOENT:
                 raise
-        log.debug('unlocked: %s', self._lockfile)
+        log.debug('unlocked: {0}'.format(self._lockfile))
 
     @property
     def is_locked(self):
@@ -334,11 +330,11 @@ class Lock:
 
     def finalize(self):
         """Unconditionally unlock the file."""
-        log.debug('finalize: %s', self._lockfile)
+        log.debug('finalize: {0}'.format(self._lockfile))
         self.unlock(unconditionally=True)
 
     def __del__(self):
-        log.debug('__del__: %s', self._lockfile)
+        log.debug('__del__: {0}'.format(self._lockfile))
         if self._owned:
             self.finalize()
 
@@ -371,8 +367,7 @@ class Lock:
         # Find out current claim's file name
         winner = self._read()
         # Now twiddle ours to the given pid.
-        self._claimfile = '%s.%s.%d' % (
-            self._lockfile, socket.getfqdn(), pid)
+        self._set_claimfile(pid)
         # Create a hard link from the global lock file to the claim file.
         # This actually does things in reverse order of normal operation
         # because we know that lockfile exists, and claimfile better not!
@@ -386,22 +381,22 @@ class Lock:
         os.unlink(winner)
         # And do some sanity checks
         assert self._linkcount == 2, (
-            'Unexpected link count: wanted 2, got %d' % self._linkcount)
+            'Unexpected link count: wanted 2, got {0:d}'.format(
+                self._linkcount))
         assert self.is_locked, 'Expected to be locked'
-        log.debug('transferred the lock: %s', self._lockfile)
+        log.debug('transferred the lock: {0}'.format(self._lockfile))
 
     def take_possession(self):
         """Take possession of a lock from another process.
 
         See `transfer_to()` for more information.
         """
-        self._claimfile = '%s.%s.%d' % (
-            self._lockfile, socket.getfqdn(), os.getpid())
+        self._set_claimfile()
         # Wait until the linkcount is 2, indicating the parent has completed
         # the transfer.
         while self._linkcount != 2 or self._read() != self._claimfile:
             time.sleep(0.25)
-        log.debug('took possession of the lock: %s', self._lockfile)
+        log.debug('took possession of the lock: {0}'.format(self._lockfile))
 
     def disown(self):
         """Disown this lock.
@@ -410,6 +405,21 @@ class Lock:
         """
         self._owned = False
 
+    def _set_claimfile(self, pid=None):
+        """Set the _claimfile private variable."""
+        if pid is None:
+            pid = os.getpid()
+        # Calculate a hard link file name that will be used to lay claim to
+        # the lock.  We need to watch out for two Lock objects in the same
+        # process pointing to the same lock file.  Without this, if you lock
+        # lf1 and do not lock lf2, lf2.locked() will still return True.
+        self._claimfile = SEP.join((
+            self._lockfile,
+            socket.getfqdn(),
+            str(pid),
+            str(random.randint(0, MAXINT)),
+            ))
+
     def _write(self):
         """Write our claim file's name to the claim file."""
         # Make sure it's group writable
@@ -419,19 +429,37 @@ class Lock:
         finally:
             fp.close()
 
+    @property
+    def retry_errnos(self):
+        """The set of errno values that cause a read retry."""
+        return self._retry_errnos[:]
+
+    @retry_errnos.setter
+    def retry_errnos(self, errnos):
+        self._retry_errnos = []
+        self._retry_errnos.extend(errnos)
+
+    @retry_errnos.deleter
+    def retry_errnos(self):
+        self._retry_errnos = []
+
     def _read(self):
         """Read the contents of our lock file.
 
         :return: The contents of the lock file or None if the lock file does
             not exist.
         """
-        try:
-            with open(self._lockfile) as fp:
-                return fp.read()
-        except EnvironmentError as error:
-            if error.errno != errno.ENOENT:
-                raise
-            return None
+        while True:
+            try:
+                with open(self._lockfile) as fp:
+                    return fp.read()
+            except EnvironmentError as error:
+                if error.errno in self._retry_errnos:
+                    self._sleep()
+                elif error.errno != errno.ENOENT:
+                    raise
+                else:
+                    return None
 
     def _touch(self, filename=None):
         """Touch the claim file into the future.
